"""LCM type definitions
This file automatically generated by lcm.
DO NOT MODIFY BY HAND!!!!
"""


from io import BytesIO
import struct

class lcmt_planar_gripper_finger_face_assignment(object):
    """ Communicates the finger-face assignment info for a specified finger. """

    __slots__ = ["utime", "finger_name", "brick_face_name", "p_BoBq_B", "is_in_contact"]

    __typenames__ = ["int64_t", "string", "string", "double", "boolean"]

    __dimensions__ = [None, None, None, [2], None]

    def __init__(self):
        self.utime = 0
        """
        in microseconds
        LCM Type: int64_t
        """

        self.finger_name = ""
        """
        Finger name: {finger1, finger2, finger3}
        LCM Type: string
        """

        self.brick_face_name = ""
        """
        Brick face name: {PosY, NegY, PosZ, NegZ}
        LCM Type: string
        """

        self.p_BoBq_B = [ 0.0 for dim0 in range(2) ]
        """
        The contact point if in contact, or the proximity witness point if not in
        contact. It is position vector from the brick's body B's origin (Bo) to a
        point Bq (a point of B), expressed in B's frame.
        LCM Type: double[2]
        """

        self.is_in_contact = False
        """
        {y-coordinate, z-coordinate}
        A boolean that indicates whether this finger is in contact with the
        specified brick face. If true, the finger is in contact and p_BoBq_B
        indicates the actual contact point. If false, the finger is not in contact
        and p_BoBq_B indicates the witness point to a proximity query.
        LCM Type: boolean
        """


    def encode(self):
        buf = BytesIO()
        buf.write(lcmt_planar_gripper_finger_face_assignment._get_packed_fingerprint())
        self._encode_one(buf)
        return buf.getvalue()

    def _encode_one(self, buf):
        buf.write(struct.pack(">q", self.utime))
        __finger_name_encoded = self.finger_name.encode('utf-8')
        buf.write(struct.pack('>I', len(__finger_name_encoded)+1))
        buf.write(__finger_name_encoded)
        buf.write(b"\0")
        __brick_face_name_encoded = self.brick_face_name.encode('utf-8')
        buf.write(struct.pack('>I', len(__brick_face_name_encoded)+1))
        buf.write(__brick_face_name_encoded)
        buf.write(b"\0")
        buf.write(struct.pack('>2d', *self.p_BoBq_B[:2]))
        buf.write(struct.pack(">b", self.is_in_contact))

    @staticmethod
    def decode(data: bytes):
        if hasattr(data, 'read'):
            buf = data
        else:
            buf = BytesIO(data)
        if buf.read(8) != lcmt_planar_gripper_finger_face_assignment._get_packed_fingerprint():
            raise ValueError("Decode error")
        return lcmt_planar_gripper_finger_face_assignment._decode_one(buf)

    @staticmethod
    def _decode_one(buf):
        self = lcmt_planar_gripper_finger_face_assignment()
        self.utime = struct.unpack(">q", buf.read(8))[0]
        __finger_name_len = struct.unpack('>I', buf.read(4))[0]
        self.finger_name = buf.read(__finger_name_len)[:-1].decode('utf-8', 'replace')
        __brick_face_name_len = struct.unpack('>I', buf.read(4))[0]
        self.brick_face_name = buf.read(__brick_face_name_len)[:-1].decode('utf-8', 'replace')
        self.p_BoBq_B = struct.unpack('>2d', buf.read(16))
        self.is_in_contact = bool(struct.unpack('b', buf.read(1))[0])
        return self

    @staticmethod
    def _get_hash_recursive(parents):
        if lcmt_planar_gripper_finger_face_assignment in parents: return 0
        tmphash = (0xc4fe83a1c182b84d) & 0xffffffffffffffff
        tmphash  = (((tmphash<<1)&0xffffffffffffffff) + (tmphash>>63)) & 0xffffffffffffffff
        return tmphash
    _packed_fingerprint = None

    @staticmethod
    def _get_packed_fingerprint():
        if lcmt_planar_gripper_finger_face_assignment._packed_fingerprint is None:
            lcmt_planar_gripper_finger_face_assignment._packed_fingerprint = struct.pack(">Q", lcmt_planar_gripper_finger_face_assignment._get_hash_recursive([]))
        return lcmt_planar_gripper_finger_face_assignment._packed_fingerprint

    def get_hash(self):
        """Get the LCM hash of the struct"""
        return struct.unpack(">Q", lcmt_planar_gripper_finger_face_assignment._get_packed_fingerprint())[0]




"""Provides utilities to aid in scalar type conversion."""

from functools import partial

from pydrake.autodiffutils import AutoDiffXd
from pydrake.common import pretty_class_name
from pydrake.common.cpp_template import (
    TemplateClass,
    TemplateMethod,
    _get_module_from_stack,
)
from pydrake.symbolic import Expression
from pydrake.systems.framework import (
    LeafSystem_,
    SystemScalarConverter,
)


def _get_conversion_pairs(T_list):
    # Take subset from supported conversions.
    T_pairs = []
    for T, U in SystemScalarConverter.SupportedConversionPairs:
        if T in T_list and U in T_list:
            T_pairs.append((T, U))
    return T_pairs


class TemplateSystem(TemplateClass):
    """Defines templated systems enabling scalar type conversion in Python.

    This differs from ``TemplateClass`` in that (a) the template must specify
    its parameters at construction time and (b) `.define` is overridden to
    allow defining ``f(T)`` for convenience.

    Any class that is added must:

    * Not define ``__init__``, as this will be overridden.
    * Define ``_construct(self, <args>, converter=None, <kwargs>)`` and
      ``_construct_copy(self, other, converter=None)`` instead. ``converter``
      must be present as an argument to ensure that it propagates properly.

    If any of these constraints are violated, then an error will be thrown
    at the time of the first class instantiation.

    Example:
        ::

            @TemplateSystem.define("MySystem_")
            def MySystem_(T):

                class Impl(LeafSystem_[T]):
                    def _construct(self, value, converter=None):
                        LeafSystem_[T].__init__(self, converter=converter)
                        self.value = value

                    def _construct_copy(self, other, converter=None):
                        Impl._construct(self, other.value, converter=converter)

                return Impl

            MySystem = MySystem_[None]  # Default instantiation.

    Things to note:

    * When defining ``_construct_copy``, if you are delegating to
      ``_construct`` within the same class, you should use
      ``Impl._construct(self, ...)``; if you use ``self._construct``, then you
      may get a child class's constructor if you are inheriting.
    * If you are delegating construction to a parent Python class for both
      constructors, use the parent class's ``__init__`` method, not its
      ``_construct`` or ``_construct_copy`` methods.
    * ``converter`` should always be non-None, as guaranteed by the
      overriding ``__init__``. We use ``converter=None`` to imply it should be
      positional, since Python2 does not have keyword-only arguments.
    """

    # TODO(eric.cousineau): Figure out if there is a way to avoid needing to
    # pass around converters in user code, avoiding the need to have Python
    # users deal with `SystemScalarConverter`.
    def __init__(self, name, T_list=None, T_pairs=None, scope=None):
        """Constructs ``TemplateSystem``.

        Args:
            T_list: List of T's that the given system supports. By default, it
                is all types supported by `LeafSystem`.
            T_pairs: List of pairs, (T, U), defining a conversion from a
                scalar type of U to T. If None, this will use all possible
                pairs that the Python bindings of ``SystemScalarConverter``
                support.
            scope: Defining ``scope``, per ``TemplateClass``'s constructor.
        """
        if scope is None:
            scope = _get_module_from_stack()
        TemplateClass.__init__(self, name, scope=scope)

        # Check scalar types and conversions, using defaults if unspecified.
        if T_list is None:
            T_list = SystemScalarConverter.SupportedScalars
        for T in T_list:
            assert T in SystemScalarConverter.SupportedScalars, (
                f"Type {T} is not a supported scalar type"
            )
        if T_pairs is None:
            T_pairs = _get_conversion_pairs(T_list)
        for T_pair in T_pairs:
            T, U = T_pair
            assert T in T_list and U in T_list, (
                f"Conversion {T_pair} is not in the original parameter list"
            )
            assert T_pair in SystemScalarConverter.SupportedConversionPairs, (
                f"Conversion {T_pair} is not supported"
            )

        self._T_list = list(T_list)
        self._T_pairs = list(T_pairs)
        self._converter = self._make_converter()

    @classmethod
    def define(
        cls, name, T_list=None, T_pairs=None, *args, scope=None, **kwargs
    ):
        """Provides a decorator which can be used define a scalar-type
        convertible System as a template.

        The decorated function must be of the form ``f(T)``, which returns a
        class which will be the instantiation for type ``T`` of the given
        template.

        Args:
            name: Name of the system template. This should match the name of
                the object being decorated.
            T_list: See ``__init__`` for more information.
            T_pairs: See ``__init__`` for more information.
            args, kwargs: These are passed to the constructor of
                ``TemplateSystem``.
        """
        if scope is None:
            scope = _get_module_from_stack()
        template = cls(name, T_list, T_pairs, *args, scope=scope, **kwargs)
        param_list = [(T,) for T in template._T_list]

        def decorator(instantiation_func):
            def wrapped(param):
                (T,) = param
                return instantiation_func(T)

            template.add_instantiations(wrapped, param_list)
            return template

        return decorator

    def _on_add(self, param, cls, skip_rename):
        TemplateClass._on_add(self, param, cls, skip_rename)
        (T,) = param

        # Check that the user has not defined `__init__`, and has defined
        # `_construct` and `_construct_copy`.
        if not issubclass(cls, LeafSystem_[T]):
            raise RuntimeError(
                f"{pretty_class_name(cls)} must inherit from {LeafSystem_[T]}"
            )

        # Use the immediate `__dict__`, rather than querying the attributes, so
        # that we don't get spillover from inheritance.
        d = cls.__dict__
        no_init = "__init__" not in d
        has_construct = "_construct" in d
        has_copy = "_construct_copy" in d
        if not no_init:
            raise RuntimeError(
                f"{pretty_class_name(cls)} defines `__init__`, but should not. "
                "Please implement `_construct` and `_construct_copy` instead."
            )
        if not has_construct:
            raise RuntimeError(
                f"{pretty_class_name(cls)} does not define `_construct`. "
                "Please ensure this is defined."
            )
        if not has_copy:
            raise RuntimeError(
                f"{pretty_class_name(cls)} does not define `_construct_copy`. "
                "Please ensure this is defined."
            )

        # Patch `__init__`.
        template = self

        def system_init(self, *args, **kwargs):
            converter = None
            if "converter" in kwargs:
                converter = kwargs.pop("converter")
            if converter is None:
                # Use default converter.
                converter = template._converter
            if template._check_if_copying(self, *args, **kwargs):
                other = args[0]
                cls._construct_copy(self, other, converter=converter)
            else:
                cls._construct(self, *args, converter=converter, **kwargs)

        cls.__init__ = system_init

        # Patch the scalar-conversion functions (only when called from Python,
        # not when called from C++) to return the Python type instead of the
        # WrappedSystem shim.
        cls.ToScalarType = TemplateMethod(cls=cls, name="ToScalarType")
        cls.ToScalarTypeMaybe = TemplateMethod(
            cls=cls, name="ToScalarTypeMaybe"
        )
        for U in SystemScalarConverter.SupportedScalars:
            new_method = template._make_new_to_scalar_type_method(
                T=U, U=T, maybe=False
            )
            new_method_maybe = template._make_new_to_scalar_type_method(
                T=U, U=T, maybe=True
            )
            if U == AutoDiffXd:
                cls.ToAutoDiffXd = new_method
                cls.ToAutoDiffXdMaybe = new_method_maybe
            elif U == Expression:
                cls.ToSymbolic = new_method
                cls.ToSymbolicMaybe = new_method_maybe
            cls.ToScalarType.add_instantiation(
                param=[U],
                instantiation=new_method,
            )
            cls.ToScalarTypeMaybe.add_instantiation(
                param=[U],
                instantiation=new_method_maybe,
            )

        return cls

    def _check_if_copying(self, obj, *args, **kwargs):
        # Checks if a function signature implies a copy constructor.
        # Since this is called after `converter` has been removed from
        # `kwargs`, we just ensure that there are no additional arguments.
        if len(args) == 1 and len(kwargs) == 0:
            if self.is_subclass_of_instantiation(type(args[0])):
                return True
        return False

    def _make(self, T, U, system_U):
        # Converts system_U (of scalar type U) to an instance of scalar type
        # T. This should mirror the logic in
        # `system_scalar_converter_internal::Make` under the file
        # `system_scalar_converter.h`.
        assert isinstance(system_U, self[U])
        result_T = self[T](system_U)
        result_T.set_name(system_U.get_name())
        return result_T

    def _make_converter(self):
        # Creates system scalar converter for the template class.
        converter = SystemScalarConverter()
        # N.B. This does not directly instantiate the template; it is deferred
        # to when the conversion is called.
        for T, U in self._T_pairs:
            conversion = partial(self._make, T, U)
            converter._AddConstructor[T, U](conversion)
        return converter

    def _make_new_to_scalar_type_method(self, *, T, U, maybe):
        # Helper for _on_add for replacing the ToScalarType{Maybe}_ methods
        # with pure-python implementations that avoid C++ lifetime challenges.
        # U is the "from" type (the type of the System being called).
        # T is the "to" type (the type of System to return).
        def _to_scalar_type(system_U):
            converter = system_U.get_system_scalar_converter()
            if not converter.IsConvertible[U, T]():
                if maybe:
                    return None
                raise RuntimeError(
                    f"System {system_U.GetSystemPathname()} "
                    f"of type {type(system_U)} "
                    f"does not support scalar conversion to type {T}"
                )
            result = self._make(T=T, U=U, system_U=system_U)
            result._HandlePostConstructionScalarConversion[U](system_U)
            return result

        _to_scalar_type.T = T
        _to_scalar_type.U = U
        _to_scalar_type.maybe = maybe
        return _to_scalar_type



"""LCM type definitions
This file automatically generated by lcm.
DO NOT MODIFY BY HAND!!!!
"""


from io import BytesIO
import struct

class lcmt_iiwa_status(object):

    __slots__ = ["utime", "num_joints", "joint_position_measured", "joint_velocity_estimated", "joint_position_commanded", "joint_position_ipo", "joint_torque_measured", "joint_torque_commanded", "joint_torque_external"]

    __typenames__ = ["int64_t", "int32_t", "double", "double", "double", "double", "double", "double", "double"]

    __dimensions__ = [None, None, ["num_joints"], ["num_joints"], ["num_joints"], ["num_joints"], ["num_joints"], ["num_joints"], ["num_joints"]]

    def __init__(self):
        self.utime = 0
        """
        The timestamp in microseconds.
        LCM Type: int64_t
        """

        self.num_joints = 0
        """ LCM Type: int32_t """
        self.joint_position_measured = []
        """
        From FRI documentation: "The currently measured joint positions of the
        robot in radians."
        LCM Type: double[num_joints]
        """

        self.joint_velocity_estimated = []
        """
        The FRI driver does not provide velocity; we estimate it in our driver via
        a low-pass filter.  Units are radians / sec.
        LCM Type: double[num_joints]
        """

        self.joint_position_commanded = []
        """
        From FRI documentation: "The last commanded joint positions of the robot in
        radians."
        LCM Type: double[num_joints]
        """

        self.joint_position_ipo = []
        """
        From FRI documentation:
        "The joint positions commanded by the interpolator in radians. When
        commanding a motion overlay in your robot application, this method will
        give access to the joint positions currently commanded by the motion
        interpolator.  This method will return NULL during monitoring mode."
        
        The Kuka motion interpolated code is a black-box to us, so we typically do
        not try to model/simulate this signal.
        LCM Type: double[num_joints]
        """

        self.joint_torque_measured = []
        """
        From FRI documentation: "The currently measured joint torques of the robot
        in Nm."
        
        This appears to be the raw measurement of the torque sensors, which is
        attempting to track joint_torque_commanded.
        
        This is *negative* w.r.t. the conventions outlined in manipulation/README.
        LCM Type: double[num_joints]
        """

        self.joint_torque_commanded = []
        """
        From FRI documentation: "The last commanded joint torques of the robot in
        Nm."
        
        This appears to be most similar to the torque input to multibody plant.
        This is *negative* w.r.t. the conventions outlined in manipulation/README.
        LCM Type: double[num_joints]
        """

        self.joint_torque_external = []
        """
        From FRI documentation: "The currently measured external joint torques of
        the robot in Nm.  The external torques corresponds to the measured torques
        when removing the torques induced by the robot itself."
        
        This appears to be the contact forces (in joint coordinates) as well as any
        residuals from modeling errors (as computed by the onboard Kuka inverse
        dynamics model, which is a black-box to us). Recall that the inertia of
        the tool is included (potentially very approximately) in the onboard Kuka
        model, so long as a tool is defined in the active Sunrise project. You can
        use the Kuka pendant to teach the tool inertia.
        
        This is correct w.r.t. the conventions outlined in manipulation/README.
        LCM Type: double[num_joints]
        """


    def encode(self):
        buf = BytesIO()
        buf.write(lcmt_iiwa_status._get_packed_fingerprint())
        self._encode_one(buf)
        return buf.getvalue()

    def _encode_one(self, buf):
        buf.write(struct.pack(">qi", self.utime, self.num_joints))
        buf.write(struct.pack('>%dd' % self.num_joints, *self.joint_position_measured[:self.num_joints]))
        buf.write(struct.pack('>%dd' % self.num_joints, *self.joint_velocity_estimated[:self.num_joints]))
        buf.write(struct.pack('>%dd' % self.num_joints, *self.joint_position_commanded[:self.num_joints]))
        buf.write(struct.pack('>%dd' % self.num_joints, *self.joint_position_ipo[:self.num_joints]))
        buf.write(struct.pack('>%dd' % self.num_joints, *self.joint_torque_measured[:self.num_joints]))
        buf.write(struct.pack('>%dd' % self.num_joints, *self.joint_torque_commanded[:self.num_joints]))
        buf.write(struct.pack('>%dd' % self.num_joints, *self.joint_torque_external[:self.num_joints]))

    @staticmethod
    def decode(data: bytes):
        if hasattr(data, 'read'):
            buf = data
        else:
            buf = BytesIO(data)
        if buf.read(8) != lcmt_iiwa_status._get_packed_fingerprint():
            raise ValueError("Decode error")
        return lcmt_iiwa_status._decode_one(buf)

    @staticmethod
    def _decode_one(buf):
        self = lcmt_iiwa_status()
        self.utime, self.num_joints = struct.unpack(">qi", buf.read(12))
        self.joint_position_measured = struct.unpack('>%dd' % self.num_joints, buf.read(self.num_joints * 8))
        self.joint_velocity_estimated = struct.unpack('>%dd' % self.num_joints, buf.read(self.num_joints * 8))
        self.joint_position_commanded = struct.unpack('>%dd' % self.num_joints, buf.read(self.num_joints * 8))
        self.joint_position_ipo = struct.unpack('>%dd' % self.num_joints, buf.read(self.num_joints * 8))
        self.joint_torque_measured = struct.unpack('>%dd' % self.num_joints, buf.read(self.num_joints * 8))
        self.joint_torque_commanded = struct.unpack('>%dd' % self.num_joints, buf.read(self.num_joints * 8))
        self.joint_torque_external = struct.unpack('>%dd' % self.num_joints, buf.read(self.num_joints * 8))
        return self

    @staticmethod
    def _get_hash_recursive(parents):
        if lcmt_iiwa_status in parents: return 0
        tmphash = (0xf8b7dc0214255e51) & 0xffffffffffffffff
        tmphash  = (((tmphash<<1)&0xffffffffffffffff) + (tmphash>>63)) & 0xffffffffffffffff
        return tmphash
    _packed_fingerprint = None

    @staticmethod
    def _get_packed_fingerprint():
        if lcmt_iiwa_status._packed_fingerprint is None:
            lcmt_iiwa_status._packed_fingerprint = struct.pack(">Q", lcmt_iiwa_status._get_hash_recursive([]))
        return lcmt_iiwa_status._packed_fingerprint

    def get_hash(self):
        """Get the LCM hash of the struct"""
        return struct.unpack(">Q", lcmt_iiwa_status._get_packed_fingerprint())[0]




"""LCM type definitions
This file automatically generated by lcm.
DO NOT MODIFY BY HAND!!!!
"""


from io import BytesIO
import struct

import drake

class lcmt_planar_gripper_finger_face_assignments(object):
    """
    Communicates the finger-face assignments for each finger.
    of the planar-gripper.
    """

    __slots__ = ["utime", "num_fingers", "finger_face_assignments"]

    __typenames__ = ["int64_t", "int32_t", "drake.lcmt_planar_gripper_finger_face_assignment"]

    __dimensions__ = [None, None, ["num_fingers"]]

    def __init__(self):
        self.utime = 0
        """
        in microseconds
        LCM Type: int64_t
        """

        self.num_fingers = 0
        """ LCM Type: int32_t """
        self.finger_face_assignments = []
        """ LCM Type: drake.lcmt_planar_gripper_finger_face_assignment[num_fingers] """

    def encode(self):
        buf = BytesIO()
        buf.write(lcmt_planar_gripper_finger_face_assignments._get_packed_fingerprint())
        self._encode_one(buf)
        return buf.getvalue()

    def _encode_one(self, buf):
        buf.write(struct.pack(">qi", self.utime, self.num_fingers))
        for i0 in range(self.num_fingers):
            assert self.finger_face_assignments[i0]._get_packed_fingerprint() == drake.lcmt_planar_gripper_finger_face_assignment._get_packed_fingerprint()
            self.finger_face_assignments[i0]._encode_one(buf)

    @staticmethod
    def decode(data: bytes):
        if hasattr(data, 'read'):
            buf = data
        else:
            buf = BytesIO(data)
        if buf.read(8) != lcmt_planar_gripper_finger_face_assignments._get_packed_fingerprint():
            raise ValueError("Decode error")
        return lcmt_planar_gripper_finger_face_assignments._decode_one(buf)

    @staticmethod
    def _decode_one(buf):
        self = lcmt_planar_gripper_finger_face_assignments()
        self.utime, self.num_fingers = struct.unpack(">qi", buf.read(12))
        self.finger_face_assignments = []
        for i0 in range(self.num_fingers):
            self.finger_face_assignments.append(drake.lcmt_planar_gripper_finger_face_assignment._decode_one(buf))
        return self

    @staticmethod
    def _get_hash_recursive(parents):
        if lcmt_planar_gripper_finger_face_assignments in parents: return 0
        newparents = parents + [lcmt_planar_gripper_finger_face_assignments]
        tmphash = (0x777a098ee2c75e9c+ drake.lcmt_planar_gripper_finger_face_assignment._get_hash_recursive(newparents)) & 0xffffffffffffffff
        tmphash  = (((tmphash<<1)&0xffffffffffffffff) + (tmphash>>63)) & 0xffffffffffffffff
        return tmphash
    _packed_fingerprint = None

    @staticmethod
    def _get_packed_fingerprint():
        if lcmt_planar_gripper_finger_face_assignments._packed_fingerprint is None:
            lcmt_planar_gripper_finger_face_assignments._packed_fingerprint = struct.pack(">Q", lcmt_planar_gripper_finger_face_assignments._get_hash_recursive([]))
        return lcmt_planar_gripper_finger_face_assignments._packed_fingerprint

    def get_hash(self):
        """Get the LCM hash of the struct"""
        return struct.unpack(">Q", lcmt_planar_gripper_finger_face_assignments._get_packed_fingerprint())[0]




import numpy
import pydrake.lcm
import pydrake.multibody.parsing
import pydrake.multibody.tree
import pydrake.systems.framework
import pydrake.systems.lcm
import pydrake.systems.primitives
import types
from typing import Callable, ClassVar, overload

ApplyDriverConfigs: Callable
PandaControlMode: types.SimpleNamespace
kIiwaArmNumJoints: int
kIiwaLcmStatusPeriod: float
kPandaArmNumJoints: int

class IiwaCommandReceiver(pydrake.systems.framework.LeafSystem):
    def __init__(self, num_joints: int = ..., control_mode: IiwaControlMode = ...) -> None: ...
    def get_commanded_position_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_commanded_torque_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_message_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_position_measured_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_time_output_port(self) -> pydrake.systems.framework.OutputPort: ...

class IiwaCommandSender(pydrake.systems.framework.LeafSystem):
    def __init__(self, num_joints: int = ..., control_mode: IiwaControlMode = ...) -> None: ...
    def get_position_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_time_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_torque_input_port(self) -> pydrake.systems.framework.InputPort: ...

class IiwaControlMode:
    __members__: ClassVar[dict] = ...  # read-only
    __entries: ClassVar[dict] = ...
    kPositionAndTorque: ClassVar[IiwaControlMode] = ...
    kPositionOnly: ClassVar[IiwaControlMode] = ...
    kTorqueOnly: ClassVar[IiwaControlMode] = ...
    def __init__(self, value: int) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __ne__(self, other: object) -> bool: ...
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

class IiwaDriver:
    __fields__: ClassVar[tuple] = ...  # read-only
    arm_child_frame_name: str | None
    control_mode: str
    desired_kp_gains: numpy.ndarray[numpy.float64[m, 1]] | None
    ext_joint_filter_tau: float
    gripper_parent_frame_name: str | None
    hand_model_name: str
    lcm_bus: str
    lcm_status_period: float
    def __init__(self, **kwargs) -> None: ...
    def __copy__(self) -> IiwaDriver: ...
    def __deepcopy__(self, arg0: dict) -> IiwaDriver: ...

class IiwaStatusReceiver(pydrake.systems.framework.LeafSystem):
    def __init__(self, num_joints: int = ...) -> None: ...
    def get_position_commanded_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_position_measured_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_time_measured_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_torque_commanded_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_torque_external_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_torque_measured_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_velocity_estimated_output_port(self) -> pydrake.systems.framework.OutputPort: ...

class IiwaStatusSender(pydrake.systems.framework.LeafSystem):
    def __init__(self, num_joints: int = ...) -> None: ...
    def get_position_commanded_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_position_measured_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_time_measured_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_torque_commanded_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_torque_external_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_torque_measured_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_velocity_estimated_input_port(self) -> pydrake.systems.framework.InputPort: ...

class PandaCommandReceiver(pydrake.systems.framework.LeafSystem):
    def __init__(self, num_joints: int, control_mode: int) -> None: ...
    def LatchInitialPosition(self, context: pydrake.systems.framework.Context) -> None: ...
    def get_commanded_position_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_commanded_torque_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_commanded_velocity_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_message_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_position_measured_input_port(self) -> pydrake.systems.framework.InputPort: ...

class PandaCommandSender(pydrake.systems.framework.LeafSystem):
    def __init__(self, num_joints: int, control_mode: int) -> None: ...
    def get_position_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_torque_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_velocity_input_port(self) -> pydrake.systems.framework.InputPort: ...

class PandaStatusReceiver(pydrake.systems.framework.LeafSystem):
    def __init__(self, num_joints: int = ...) -> None: ...
    def get_acceleration_commanded_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_position_commanded_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_position_measured_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_torque_commanded_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_torque_external_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_torque_measured_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_velocity_commanded_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_velocity_measured_output_port(self) -> pydrake.systems.framework.OutputPort: ...

class PandaStatusSender(pydrake.systems.framework.LeafSystem):
    def __init__(self, num_joints: int = ...) -> None: ...
    def get_acceleration_commanded_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_position_commanded_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_position_measured_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_torque_commanded_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_torque_external_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_torque_measured_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_velocity_commanded_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_velocity_measured_input_port(self) -> pydrake.systems.framework.InputPort: ...

class SchunkWsgCommandReceiver(pydrake.systems.framework.LeafSystem):
    def __init__(self, initial_position: float = ..., initial_force: float = ...) -> None: ...
    def get_force_limit_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_position_output_port(self) -> pydrake.systems.framework.OutputPort: ...

class SchunkWsgCommandSender(pydrake.systems.framework.LeafSystem):
    def __init__(self, default_force_limit: float = ...) -> None: ...
    def get_command_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_force_limit_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_position_input_port(self) -> pydrake.systems.framework.InputPort: ...

class SchunkWsgController(pydrake.systems.framework.Diagram):
    def __init__(self, kp: float = ..., ki: float = ..., kd: float = ...) -> None: ...

class SchunkWsgDriver:
    __fields__: ClassVar[tuple] = ...  # read-only
    lcm_bus: str
    pid_gains: numpy.ndarray[numpy.float64[3, 1]]
    def __init__(self, **kwargs) -> None: ...
    def __copy__(self) -> SchunkWsgDriver: ...
    def __deepcopy__(self, arg0: dict) -> SchunkWsgDriver: ...

class SchunkWsgPositionController(pydrake.systems.framework.Diagram):
    def __init__(self, time_step: float = ..., kp_command: float = ..., kd_command: float = ..., kp_constraint: float = ..., kd_constraint: float = ..., default_force_limit: float = ...) -> None: ...
    def get_desired_position_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_force_limit_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_generalized_force_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_grip_force_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_state_input_port(self) -> pydrake.systems.framework.InputPort: ...

class SchunkWsgStatusReceiver(pydrake.systems.framework.LeafSystem):
    def __init__(self) -> None: ...
    def get_force_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_state_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_status_input_port(self) -> pydrake.systems.framework.InputPort: ...

class SchunkWsgStatusSender(pydrake.systems.framework.LeafSystem):
    def __init__(self) -> None: ...
    def get_force_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_state_input_port(self) -> pydrake.systems.framework.InputPort: ...

class SchunkWsgTrajectoryGenerator(pydrake.systems.framework.LeafSystem):
    def __init__(self, input_size: int, position_index: int, use_force_limit: bool = ...) -> None: ...
    def get_desired_position_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_force_limit_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_max_force_output_port(self) -> pydrake.systems.framework.OutputPort: ...
    def get_state_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def get_target_output_port(self) -> pydrake.systems.framework.OutputPort: ...

class SimIiwaDriver(pydrake.systems.framework.Diagram):
    def __init__(self, driver_config: IiwaDriver, controller_plant) -> None: ...
    @staticmethod
    def AddToBuilder(builder: pydrake.systems.framework.DiagramBuilder, plant, iiwa_instance: pydrake.multibody.tree.ModelInstanceIndex, driver_config: IiwaDriver, controller_plant) -> pydrake.systems.framework.System: ...

class ZeroForceDriver:
    __fields__: ClassVar[tuple] = ...  # read-only
    def __init__(self, **kwargs) -> None: ...
    def __copy__(self) -> ZeroForceDriver: ...
    def __deepcopy__(self, arg0: dict) -> ZeroForceDriver: ...

@overload
def ApplyDriverConfig(driver_config: IiwaDriver, model_instance_name: str, sim_plant, models_from_directives: dict[str, pydrake.multibody.parsing.ModelInstanceInfo], lcms: pydrake.systems.lcm.LcmBuses, builder: pydrake.systems.framework.DiagramBuilder) -> None: ...
@overload
def ApplyDriverConfig(driver_config: SchunkWsgDriver, model_instance_name: str, sim_plant, models_from_directives: dict[str, pydrake.multibody.parsing.ModelInstanceInfo], lcms: pydrake.systems.lcm.LcmBuses, builder: pydrake.systems.framework.DiagramBuilder) -> None: ...
@overload
def ApplyDriverConfig(driver_config: ZeroForceDriver, model_instance_name: str, sim_plant, models_from_directives: dict[str, pydrake.multibody.parsing.ModelInstanceInfo], lcms: pydrake.systems.lcm.LcmBuses, builder: pydrake.systems.framework.DiagramBuilder) -> None: ...
def ApplyNamedPositionsAsDefaults(input: dict[str, dict[str, numpy.ndarray[numpy.float64[m, 1]]]], plant) -> None: ...
def BuildIiwaControl(builder: pydrake.systems.framework.DiagramBuilder, lcm: pydrake.lcm.DrakeLcmInterface, plant, iiwa_instance: pydrake.multibody.tree.ModelInstanceIndex, driver_config: IiwaDriver, controller_plant) -> None: ...
def BuildSchunkWsgControl(plant, wsg_instance: pydrake.multibody.tree.ModelInstanceIndex, lcm: pydrake.lcm.DrakeLcmInterface, builder: pydrake.systems.framework.DiagramBuilder, pid_gains: numpy.ndarray[numpy.float64[3, 1]] | None = ...) -> None: ...
def GetSchunkWsgOpenPosition() -> numpy.ndarray[numpy.float64[m, 1]]: ...
def MakeMultibodyForceToWsgForceSystem() -> pydrake.systems.framework.VectorSystem: ...
def MakeMultibodyStateToWsgStateSystem() -> pydrake.systems.primitives.MatrixGain: ...
def ParseIiwaControlMode(control_mode: str) -> IiwaControlMode: ...
def get_iiwa_max_joint_velocities() -> numpy.ndarray[numpy.float64[m, 1]]: ...
def position_enabled(control_mode: IiwaControlMode) -> bool: ...
def torque_enabled(control_mode: IiwaControlMode) -> bool: ...



import numpy as _np


def _resolve_array_type(x):
    # Resolves the scalar type for a given array.
    assert isinstance(x, _np.ndarray), type(x)
    assert x.size != 0
    if x.dtype != object:
        if x.dtype == float or x.dtype == int:
            return float
        else:
            return x.dtype.type
    else:
        # Search array for any non-builtin and non-numpy types.
        for xi in x.flat:
            t = type(xi)
            if t.__module__ not in ("builtins", "numpy"):
                return t
        # Unable to infer type.
        return None


def _check_returned_array_type(cls_name, y, expected_type):
    # Used by CheckReturnedArrayType in C++.
    if y.size == 0:
        return
    actual_type = _resolve_array_type(y)
    expected_name = expected_type.__name__
    if actual_type is None:
        raise TypeError(
            f"When {cls_name} is called with an array of type {expected_name} "
            f"the return value must be the same type."
        )
    if actual_type is not expected_type:
        actual_name = actual_type.__name__
        raise TypeError(
            f"When {cls_name} is called with an array of type {expected_name} "
            f"the return value must be the same type, not {actual_name}."
        )



import numpy
import pydrake.geometry
import pydrake.multibody.plant
import pydrake.systems.framework
from pydrake.common.cpp_template import ContactVisualizer_ as ContactVisualizer_, JointSliders_ as JointSliders_
from typing import ClassVar

class ContactVisualizer(pydrake.systems.framework.LeafSystem):
    def __init__(self, meshcat: pydrake.geometry.Meshcat, params: ContactVisualizerParams = ...) -> None: ...
    @staticmethod
    def AddToBuilder(builder: pydrake.systems.framework.DiagramBuilder, plant: pydrake.multibody.plant.MultibodyPlant, meshcat: pydrake.geometry.Meshcat, params: ContactVisualizerParams = ...) -> ContactVisualizer: ...
    def Delete(self) -> None: ...
    def contact_results_input_port(self) -> pydrake.systems.framework.InputPort: ...
    def query_object_input_port(self) -> pydrake.systems.framework.InputPort: ...

class ContactVisualizerParams:
    __fields__: ClassVar[tuple] = ...  # read-only
    color: pydrake.geometry.Rgba
    delete_on_initialization_event: bool
    force_threshold: float
    hydro_force_color: pydrake.geometry.Rgba
    hydro_moment_color: pydrake.geometry.Rgba
    moment_threshold: float
    newton_meters_per_meter: float
    newtons_per_meter: float
    prefix: str
    publish_period: float
    radius: float
    def __init__(self, **kwargs) -> None: ...
    def __copy__(self) -> ContactVisualizerParams: ...
    def __deepcopy__(self, arg0: dict) -> ContactVisualizerParams: ...

class ContactVisualizer_𝓣AutoDiffXd𝓤(pydrake.systems.framework.LeafSystem_TAutoDiffXdU):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, meshcat: pydrake.geometry.Meshcat, params: ContactVisualizerParams = ...) -> None: ...
    @staticmethod
    def AddToBuilder(builder: pydrake.systems.framework.DiagramBuilder_TAutoDiffXdU, plant: pydrake.multibody.plant.MultibodyPlant_TAutoDiffXdU, meshcat: pydrake.geometry.Meshcat, params: ContactVisualizerParams = ...) -> ContactVisualizer_TAutoDiffXdU: ...
    def Delete(self) -> None: ...
    def contact_results_input_port(self) -> pydrake.systems.framework.InputPort_TAutoDiffXdU: ...
    def query_object_input_port(self) -> pydrake.systems.framework.InputPort_TAutoDiffXdU: ...

class JointSliders(pydrake.systems.framework.LeafSystem):
    def __init__(self, meshcat: pydrake.geometry.Meshcat, plant: pydrake.multibody.plant.MultibodyPlant, initial_value: numpy.ndarray[numpy.float64[m, 1]] | None = ..., lower_limit: None | float | numpy.ndarray[numpy.float64[m, 1]] = ..., upper_limit: None | float | numpy.ndarray[numpy.float64[m, 1]] = ..., step: None | float | numpy.ndarray[numpy.float64[m, 1]] = ..., decrement_keycodes: list[str] = ..., increment_keycodes: list[str] = ...) -> None: ...
    def Delete(self) -> None: ...
    def Run(self, diagram: pydrake.systems.framework.Diagram, timeout: float | None = ..., stop_button_keycode: str = ...) -> numpy.ndarray[numpy.float64[m, 1]]: ...
    def SetPositions(self, q: numpy.ndarray[numpy.float64[m, 1]]) -> None: ...

class JointSliders_𝓣AutoDiffXd𝓤(pydrake.systems.framework.LeafSystem_TAutoDiffXdU):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, meshcat: pydrake.geometry.Meshcat, plant: pydrake.multibody.plant.MultibodyPlant_TAutoDiffXdU, initial_value: numpy.ndarray[numpy.float64[m, 1]] | None = ..., lower_limit: None | float | numpy.ndarray[numpy.float64[m, 1]] = ..., upper_limit: None | float | numpy.ndarray[numpy.float64[m, 1]] = ..., step: None | float | numpy.ndarray[numpy.float64[m, 1]] = ..., decrement_keycodes: list[str] = ..., increment_keycodes: list[str] = ...) -> None: ...
    def Delete(self) -> None: ...
    def Run(self, diagram: pydrake.systems.framework.Diagram_TAutoDiffXdU, timeout: float | None = ..., stop_button_keycode: str = ...) -> numpy.ndarray[numpy.float64[m, 1]]: ...
    def SetPositions(self, q: numpy.ndarray[numpy.float64[m, 1]]) -> None: ...

class _HydroelasticContactVisualizer:
    def __init__(self, meshcat: pydrake.geometry.Meshcat, params: ContactVisualizerParams) -> None: ...
    def Delete(self) -> None: ...
    def Update(self, time: float, items: list[_HydroelasticContactVisualizerItem]) -> None: ...

class _HydroelasticContactVisualizerItem:
    body_A: str
    body_B: str
    centroid_W: numpy.ndarray[numpy.float64[3, 1]]
    faces: numpy.ndarray[numpy.int32[3, n]]
    force_C_W: numpy.ndarray[numpy.float64[3, 1]]
    moment_C_W: numpy.ndarray[numpy.float64[3, 1]]
    p_WV: numpy.ndarray[numpy.float64[3, n]]
    pressure: numpy.ndarray[numpy.float64[m, 1]]
    def __init__(self, body_A: str, body_B: str, centroid_W: numpy.ndarray[numpy.float64[3, 1]], force_C_W: numpy.ndarray[numpy.float64[3, 1]], moment_C_W: numpy.ndarray[numpy.float64[3, 1]], p_WV: numpy.ndarray[numpy.float64[3, n]], faces: numpy.ndarray[numpy.int32[3, n]], pressure: numpy.ndarray[numpy.float64[m, 1]]) -> None: ...

class _PointContactVisualizer:
    def __init__(self, meshcat: pydrake.geometry.Meshcat, params: ContactVisualizerParams) -> None: ...
    def Update(self, time: float, items: list[_PointContactVisualizerItem]) -> None: ...

class _PointContactVisualizerItem:
    body_A: str
    body_B: str
    contact_force: numpy.ndarray[numpy.float64[3, 1]]
    contact_point: numpy.ndarray[numpy.float64[3, 1]]
    def __init__(self, body_A: str, body_B: str, contact_force: numpy.ndarray[numpy.float64[3, 1]], contact_point: numpy.ndarray[numpy.float64[3, 1]]) -> None: ...

class _TemporaryName_N5drake9multibody7meshcat12JointSlidersINS_2ad8AutoDiffEEE(pydrake.systems.framework.LeafSystem_TAutoDiffXdU):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, meshcat: pydrake.geometry.Meshcat, plant: pydrake.multibody.plant.MultibodyPlant_TAutoDiffXdU, initial_value: numpy.ndarray[numpy.float64[m, 1]] | None = ..., lower_limit: None | float | numpy.ndarray[numpy.float64[m, 1]] = ..., upper_limit: None | float | numpy.ndarray[numpy.float64[m, 1]] = ..., step: None | float | numpy.ndarray[numpy.float64[m, 1]] = ..., decrement_keycodes: list[str] = ..., increment_keycodes: list[str] = ...) -> None: ...
    def Delete(self) -> None: ...
    def Run(self, diagram: pydrake.systems.framework.Diagram_TAutoDiffXdU, timeout: float | None = ..., stop_button_keycode: str = ...) -> numpy.ndarray[numpy.float64[m, 1]]: ...
    def SetPositions(self, q: numpy.ndarray[numpy.float64[m, 1]]) -> None: ...

class _TemporaryName_N5drake9multibody7meshcat17ContactVisualizerINS_2ad8AutoDiffEEE(pydrake.systems.framework.LeafSystem_TAutoDiffXdU):
    _original_name: ClassVar[str] = ...
    _original_qualname: ClassVar[str] = ...
    def __init__(self, meshcat: pydrake.geometry.Meshcat, params: ContactVisualizerParams = ...) -> None: ...
    @staticmethod
    def AddToBuilder(builder: pydrake.systems.framework.DiagramBuilder_TAutoDiffXdU, plant: pydrake.multibody.plant.MultibodyPlant_TAutoDiffXdU, meshcat: pydrake.geometry.Meshcat, params: ContactVisualizerParams = ...) -> ContactVisualizer_TAutoDiffXdU: ...
    def Delete(self) -> None: ...
    def contact_results_input_port(self) -> pydrake.systems.framework.InputPort_TAutoDiffXdU: ...
    def query_object_input_port(self) -> pydrake.systems.framework.InputPort_TAutoDiffXdU: ...



"""LCM type definitions
This file automatically generated by lcm.
DO NOT MODIFY BY HAND!!!!
"""


from io import BytesIO
import struct

import drake

class lcmt_image_array(object):
    """ This is used for sending and/or receiving multiple images at the same time. """

    __slots__ = ["header", "num_images", "images"]

    __typenames__ = ["drake.lcmt_header", "int32_t", "drake.lcmt_image"]

    __dimensions__ = [None, None, ["num_images"]]

    def __init__(self):
        self.header = drake.lcmt_header()
        """
        The timestamp and the frame name.
        The timestamp holds when this data is packed. It's convenient to store
        the latest timestamp among the timestamps in `images` since it is possible
        for each of `images` having different timestamps if their camera's shutters
        are not synchronized.
        The `frame_name` can be empty if each image_t in `images` contains the
        information in it.
        LCM Type: drake.lcmt_header
        """

        self.num_images = 0
        """
        The number of images.
        LCM Type: int32_t
        """

        self.images = []
        """
        An array of image_t.
        LCM Type: drake.lcmt_image[num_images]
        """


    def encode(self):
        buf = BytesIO()
        buf.write(lcmt_image_array._get_packed_fingerprint())
        self._encode_one(buf)
        return buf.getvalue()

    def _encode_one(self, buf):
        assert self.header._get_packed_fingerprint() == drake.lcmt_header._get_packed_fingerprint()
        self.header._encode_one(buf)
        buf.write(struct.pack(">i", self.num_images))
        for i0 in range(self.num_images):
            assert self.images[i0]._get_packed_fingerprint() == drake.lcmt_image._get_packed_fingerprint()
            self.images[i0]._encode_one(buf)

    @staticmethod
    def decode(data: bytes):
        if hasattr(data, 'read'):
            buf = data
        else:
            buf = BytesIO(data)
        if buf.read(8) != lcmt_image_array._get_packed_fingerprint():
            raise ValueError("Decode error")
        return lcmt_image_array._decode_one(buf)

    @staticmethod
    def _decode_one(buf):
        self = lcmt_image_array()
        self.header = drake.lcmt_header._decode_one(buf)
        self.num_images = struct.unpack(">i", buf.read(4))[0]
        self.images = []
        for i0 in range(self.num_images):
            self.images.append(drake.lcmt_image._decode_one(buf))
        return self

    @staticmethod
    def _get_hash_recursive(parents):
        if lcmt_image_array in parents: return 0
        newparents = parents + [lcmt_image_array]
        tmphash = (0x3afa7aac7daa3b4e+ drake.lcmt_header._get_hash_recursive(newparents)+ drake.lcmt_image._get_hash_recursive(newparents)) & 0xffffffffffffffff
        tmphash  = (((tmphash<<1)&0xffffffffffffffff) + (tmphash>>63)) & 0xffffffffffffffff
        return tmphash
    _packed_fingerprint = None

    @staticmethod
    def _get_packed_fingerprint():
        if lcmt_image_array._packed_fingerprint is None:
            lcmt_image_array._packed_fingerprint = struct.pack(">Q", lcmt_image_array._get_hash_recursive([]))
        return lcmt_image_array._packed_fingerprint

    def get_hash(self):
        """Get the LCM hash of the struct"""
        return struct.unpack(">Q", lcmt_image_array._get_packed_fingerprint())[0]




r"""
Utility for converting a mesh file into an SDFormat file containing a single
model.

Currently only meshes in Wavefront OBJ files are supported. Other geometry
formats will be supported in the future.

Preconditions on the OBJ:

  - The mesh is "watertight" (no cracks, no openings).
  - The mesh is closed (i.e., no sheets of triangles).

Properties of the resulting SDFormat file:

  - The model contains a single body.
  - The body and model will share a common name.
  - The mass properties are computed based on the volume enclosed by the mesh.
    (If the preconditions are not met, mass properties will nevertheless be
    computed, but they are not guaranteed to be correct).
  - The mesh will be used as both <collision> and <visual> geometries.

**Running**:

    From a Drake source build, run this as::

            bazel run //tools:mesh_to_model -- --help

    From a Drake binary release (including pip releases), run this as::

            python3 -m pydrake.multibody.mesh_to_model path/to/mesh.obj

**Model scale**:

    Drake uses meters as the unit of length. Modelling packages may frequently
    use other units (e.g. centimeters). This program will *assume* the units
    are meters. If they are not, provide the scale conversion factor to go from
    mesh units to meters. For example, if your mesh is in centimeters, use 0.01
    for the scale.

**Configuring mass**:

    The body's mass can be configured through one of two ways: specifying the
    _density_ of a homogeneous material which completely fills the enclosed
    volume of the mesh, or specifying the total _mass_ of the body. If neither
    is specified, it is assumed to have the density of water (~1000kg/m³). Only
    one of density and mass can be specified.

**Geometry frame vs body frame**:

    The mesh's vertices are measured and expressed in an arbitrary geometry
    frame G. By default, the body frame B is coincident with G (i.e.,
    X_BG = I). The two frames will always be aligned (R_BG = R_GB = I), but the
    relative positions of the origins (p_GoBo) can be modified. You can either
    request that the body origin be coincident with the computed center of mass
    or specify an arbitrary value for p_GoBo (the body's origin position
    relative to the mesh's origin).

**Packages**:

    If the mesh is used in a <visual> or <collision> element (see above) the
    mesh must be referenced via some url. The default mesh url is a simple
    relative path assuming the SDFormat file and mesh files are in the same
    directory. There are two alternatives: create/use a package at the mesh
    file location or specify a package.

        Default behavior:

            When calling::

                mesh_to_model path/to/my_models/mesh.obj

            the resulting mesh's <uri> element will look like::

                <uri>mesh.obj</uri>

            This uri resolution is not SDFormat standard but is acceptable as a
            Drake parser extension. It is your responsibility to place the mesh
            in the same folder as the sdf.

        Specify package:

            When calling::

                mesh_to_model --package=path/to/package.xml \
                    path/to/my_models/mesh.obj

            the resulting mesh's <uri> element will look like::

                <uri>package://my_package/my_models/mesh.obj</uri>

            This example assumes that the <name> field in the indicated
            package.xml is "my_package". Errors in reading the file (or the
            package name from the manifest) will cause the program to fail. The
            mesh file must live somewhere underneath the package.xml directory.

        Create/use package at mesh file location:

            When calling::

                mesh_to_model --package=auto path/to/my_models/mesh.obj

            the program will look for the existence of
            path/to/my_models/package.xml. If it doesn't exist, it will be
            created. The resulting mesh's <uri> element will look like::

                <uri>package://my_models/mesh.obj</uri>

            The package.xml file will be minimally compliant but may contain
            meaningless values (see
            http://wiki.ros.org/Manifest#catkin.2Fpackage.xml.Required_Tags).
            Any error in reading or writing the file will cause the program to
            fail.
"""

import argparse
import os
from pathlib import Path

import numpy as np

from pydrake.common import configure_logging as _configure_logging
from pydrake.multibody._mesh_model_maker import (
    MeshModelMaker as _MeshModelMaker,
)


def _CommaSeparatedXYZ(arg: str):
    """An argparse parser for an x,y,z vector."""
    x, y, z = [float(item) for item in arg.split(",")]
    return np.array([x, y, z])


# In the event that the user passes in a malformed value for --body-origin,
# this makes the error message nicer.
_CommaSeparatedXYZ.__name__ = "comma-separated triple"


def _main():
    _configure_logging()

    default_maker = _MeshModelMaker(mesh_path=None, output_dir=None)

    parser = argparse.ArgumentParser(
        description=__doc__,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--model-name",
        metavar="NAME",
        help=(
            "The optional name to assign to both model and body. If none "
            "is given, the mesh file name is used."
        ),
    )
    parser.add_argument(
        "--scale",
        default=default_maker.scale,
        type=float,
        metavar="FACTOR",
        help=(
            "The scale for scaling the mesh vertex positions to meters. "
            "Must be positive."
        ),
    )

    mass_group = parser.add_mutually_exclusive_group()
    mass_group.add_argument(
        "--density",
        default=default_maker.density,
        type=float,
        metavar="KG/M3",
        help=(
            "The density of the object in kg/m³. Only specify one of "
            "--density and --mass. If neither is specified, the default "
            "density of water is used."
        ),
    )
    mass_group.add_argument(
        "--mass",
        type=float,
        metavar="KG",
        help=(
            "The mass of the object in kg. Only specify one of --density "
            "and --mass. If neither is specified, default density of water "
            "is used."
        ),
    )

    com_group = parser.add_mutually_exclusive_group()
    com_group.add_argument(
        "--origin-at-com",
        action="store_true",
        dest="at_com",
        help=(
            "If requested, the body's origin is defined at the geometry's "
            "center of mass. Only specify one of --body-origin and "
            "--origin-at-com."
        ),
    )
    com_group.add_argument(
        "--body-origin",
        type=_CommaSeparatedXYZ,
        dest="p_GoBo",
        metavar="X,Y,Z",
        help=(
            "Specify the body origin in the mesh's canonical frame "
            "(p_GoBo_G). Only specify one of --body-origin and "
            "--origin-at-com."
        ),
    )

    parser.add_argument(
        "--package",
        type=str,
        dest="encoded_package",
        default=default_maker.encoded_package,
        metavar="ENCODING",
        help=(
            "Specify the package semantics used in the resulting SDFormat "
            "file. Options are: 'none' (default): the mesh will be "
            "referenced using a relative path in the same directory, "
            "'auto': current directory is treated as a package root and "
            "package.xml will be added if not already present), or "
            "[path to a package.xml file]: the mesh will be referenced as "
            "being in the root of that package)."
        ),
    )

    parser.add_argument(
        "--output-dir",
        type=Path,
        help=(
            "The path where the SDFormat model will be written. If not "
            "given, the file will be written in the mesh's directory. If "
            "given, the directory must already exist. The model file will "
            "share the same stem name as the mesh file."
        ),
    )

    parser.add_argument(
        "mesh_path", type=Path, help="The path to the mesh file to process."
    )

    args = parser.parse_args()

    if "BUILD_WORKSPACE_DIRECTORY" in os.environ:
        os.chdir(os.environ["BUILD_WORKING_DIRECTORY"])

    maker = _MeshModelMaker(**vars(args))
    maker.make_model()


if __name__ == "__main__":
    _main()
